%% Numerische Integration mit Matlab
%% am Beispiel einer e-Funktion
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%

clear all
close all

% Notwendige Ordner zum Matlabpfad hinzufuegen
nameFile = mfilename('fullpath');
pathFile = fileparts(nameFile);
pathEfun = fullfile(pathFile,'eFunktion');
pathInt  = fullfile(pathFile,'eulerIntegratoren');
addpath(pathEfun,pathInt)

%% Gesucht: Zeitverlauf y(t)

% Simulationszeit
t0 = 0; % [s]
t1 = 2; % [s]

% Anfangsbedingungen y(t0) 
y0 = 10; 

% Schrittweite
h = 0.2; % 0.01 0.1 0.2 0.3

%% e-Funktion
% DGL:   d/dt(y) = -a *y(t)
% exakt Lsg: y(t) = y0 * exp(-a*t)

%% exakte Loesung
texakt = simTime2vector(t0,t1,0.01);
Yexakt = eFunktionExakt(texakt,y0);

hFig = figure(1);
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'e-Funktion','Zeit [s]','$y = f(t)$')



%% Integration mit Euler-Vorwaerts
[teEeuler,YeEuler] = eEuler(@eFunktionDGL,[t0 t1],y0,h);
Yint = eFunktionExakt(teEeuler,y0);

% Vgl. Euler-Vorwaerts Zeitintegration mit exakter Loesung
hFig = figure(2);
plot(teEeuler,YeEuler,'DisplayName','Euler-Vorwaerts')
hold all
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'e-Funktion','Zeit [s]','$y = f(t)$')


% Absoluter Integrationsfehler Euler-Vorwaerts
hFig = figure(3);
plot(teEeuler,YeEuler-Yint,'DisplayName','$e_{abs} = y_{euler} - y_{exakt}$')

createFigureLabels(hFig,'absoluter Integrationsfehler Euler-Vorwaerts',...
                        'Zeit [s]',...
                        'Fehler')
                    
% Relativer Fehler Euler-Vorwaerts
hFig = figure(4);
plot(teEeuler,abs(YeEuler+100-(Yint+100))./(Yint+100)*100,'DisplayName','$e_{rel} = \frac{\|y_{euler} - y_{exakt}\|}{\|y_{exakt}\|}$')

createFigureLabels(hFig,'relativer Integrationsfehler Euler-Vorwaerts',...
                        'Zeit [s]',...
                        'Fehler [\%]')



%% Integration mit Euler-Rueckwaerts
[tiEeuler,YiEuler] = iEuler(@eFunktionDGL,[t0 t1],y0,h);
Yint = eFunktionExakt(tiEeuler,y0);

% Vgl. Euler-RueckwaertsZeitintegration mit exakter Loesung
hFig = figure(5);
plot(tiEeuler,YiEuler,'DisplayName','Euler-Rueckwaerts')
hold all
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'e-Funktion','Zeit [s]','$y = f(t)$')


% Absoluter Integrationsfehler Euler-Rueckwaerts
hFig = figure(6);
plot(tiEeuler,YiEuler-Yint,'DisplayName','$e_{abs} = y_{euler} - y_{exakt}$')

createFigureLabels(hFig,'absoluter Integrationsfehler Euler-Rueckwaerts',...
                        'Zeit [s]',...
                        'Fehler')

% Relativer Fehler Euler-Rueckwaerts
hFig = figure(7);
plot(tiEeuler,abs(YiEuler+100-(Yint+100))./(Yint+100)*100,'DisplayName','$e_{rel} = \frac{\|y_{euler} - y_{exakt}\|}{\|y_{exakt}\|}$')

createFigureLabels(hFig,'relativer Integrationsfehler Euler-Rueckwaerts',...
                        'Zeit [s]',...
                        'Fehler [\%]')

%% Integration mit ODE45 (Matlab built-in)
% explizites Runge-Kutta 4
% variable Schrittweite
t = simTime2vector(t0,t1,h);

sol = ode45(@eFunktionDGL,[t0, t1],y0);
Yruku4 = deval(t,sol);

% Vgl. ode45 Zeitintegration mit exakter Loesung
hFig = figure(8);
plot(t,Yruku4,'DisplayName','Runge-Kutta 4')
hold all
plot(texakt,Yexakt,'DisplayName','exakte Loesung')

createFigureLabels(hFig,'e-Funktion','Zeit [s]','$y = f(t)$');

% Absoluter Integrationsfehler ode45
hFig = figure(9);
plot(t,Yruku4-Yint,'DisplayName','$e_{abs} = y_{ruku4}-y_{exakt}$')

createFigureLabels(hFig,'absoluter Integrationsfehler Runge-Kutta 4',...
                        'Zeit [s]',...
                        'Fehler')

% Relativer Fehler Runge-Kutta 4
hFig = figure(10);
plot(t,abs(Yruku4+100-(Yint+100))./(Yint+100)*100,'DisplayName','$e_{rel} = \frac{\|y_{ruku4} - y_{exakt}\|}{\|y_{exakt}\|}$')

createFigureLabels(hFig,'relativer Integrationsfehler Runge-Kutta 4',...
                        'Zeit [s]',...
                        'Fehler [\%]')
                    

% Variable Schrittweitensteuerung
hFig = figure(11);
bar(diff(sol.x))
grid on
title('Automatische Schrittweitensteuerung ode45')
xlabel('Integrationsschritte')
ylabel('Schrittweite [s]')