function [K, F] = Biegelinie(nA,l,EI,q0)
%% Biegelinie
% [K, F] = Biegelinie(nA,l,EI,q0)
% Ausgabe der Matrizen K, F zur Berechnung der Knotenverschiebungen v eines
% mit Hilfe der finiten Elemente Methode diskretisierten Balkens
% K*v = F
% Biegelinie bei konstanter Biegesteifigkeit
% Einseitig feste Einspannung und konstante Streckenlast
%
% nA: Anzahl der Balkenelemente
% l: Balkenlaenge [mm]
% EI: Biegesteifigkeit [Nmm^2]
% q0: Streckenlast [N/mm]
%
% K*v = F
% K: Steifigkeitsmatrix
% v: Durchbiegung
% F: Querkraft
%
% Differenzialgleichung:
% EI v^IV = q
% Differenzenformel 1. Ordnung: y^I = 1/(2h) (-v(i-1) + v(i+1))
% Mit der Differenzenformel 4. Ordnung ergibt sich die Differenzengleichung:
% v(i-2) - 4 v(i-1) + v(i+1) + v(i+2) = q0 l^4 / (nA^4*EI)
%
% Technische Mechanik, J. Dankert, H. Dankert, Springer Vieweg, 7. Auflage
% Kapitel 18.1.2: Biegelinie bei konstanter Biegesteifigkeit, S.275-278
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%


n = nA + 5; % Anzahl der Gleichungen: nA+1 Knoten und 4 Randbedingungen

% Initialisieren der Matrizen:
K = zeros(n,n);
F = zeros(n,1);
fvalue = (q0*l^4)/(nA^4*EI); % Vorberechnung der rechten Seite

% F�llen der Matrizen
K(1,3) = 1; % v3 = 0 - Feste Einspannung
K(2,2:4) = [-1, 0, 1]; % 1. Abl fester Einspannung
K(end-1,end-4:end) = [-1 2 0 -2 1]; % Querkraftfrei am Balkenende
K(end, end-3:end-1) = [1 -2 1]; % Momentenfreiheit am Balkenende
for i = 3:1:n-2
    K(i,i-2:i+2)= [1, -4, 6, -4, 1];
    F(i) = fvalue;
end
end