function [X, tElapsed] = GaussOhnePivot(A,B,varargin)
%% LGS Loeser mit Gauss-Elimination ohne Pivotsuche vgl. A7.1
%% mit Option auf LR-Zerlegung vgl. A8.1
% [X, tElapsed] = GaussOhnePivot(A,B)
%
% Numerische Methoden der Dynamik - SS19
% Dr.-Ing. Pascal Ziegler
%
% Institut fuer Technische und Numerische Mechanik
% Prof. Dr.-Ing. Prof. E.h. P. Eberhard
%
% Autor: Andreas Hofmann, M.Sc.
%

% Default Werte
epsilon = 1e-6;
lr = false;

for h = 2:2:length(varargin)
    switch lower(varargin{h-1})
        case 'toleranz'
            if isnumeric(varargin{h})
                epsilon = varargin{h};
            else
                warning(['Eingabeoptionen pruefen!',...
                    ' Keine spezifische Toleranz definiert.',...
                    ' Berechnung mit dem Default-Wert: ',num2str(epsilon)])
            end
            
        case 'lr' % LR-Zerlegung
            if islogical(varargin{h})
                lr = varargin{h};
            else
                warning(['Eingabeoptionen pruefen!',...
                    ' Keine logischer Wert fuer die LR Option uebergeben.',...
                    ' LR-Zerlegung wird nicht durchgefuehrt: ',num2str(epsilon)])
            end           
    end
end

%% 1. Untersuchung der �bergebenen Werte
dimA = size(A); % dimA = [nZeilen nSpalten]; A(n x n)
dimB = size(B); % dimB = [nZeilen nSpalten]; B(n x q)

if dimA(1) ~= dimA(2)
    error('Matrix A ist nicht quadratisch!')
elseif dimA(1) ~= dimB(1)
    error('Dimensionen von A und b sind ungleich!')
end

n = dimA(1);
q = dimB(2);

%% 2. Gauss-Elimination ohne Pivotsuche
tStart = tic; % Start Zeitmessung

for s = 1:1:(n-1)
    if abs(A(s,s)) <= epsilon
        error('Pivotsuche erforderlich!')
    end
    
    % Elimination
    for i = (s+1):1:n
        l = A(i,s)/A(s,s);
        if lr
            A(i,s) = l;
        else
            A(i,s) = 0;
        end
        
        for j =(s+1):1:n
            A(i,j) = A(i,j)-( l * A(s,j));
        end
        
        for k = 1:1:q
            B(i,k) = B(i,k) - (l * B(s,k));
        end
    end
end
if abs(A(n,n)) <= epsilon
    error ('A ist singulaer')
end


if lr % LR-Zerlegung
    % A = L*R
    % => L*R * X = B
    % => L*Y = B ; Y = R*X
    R = triu(A);
    L = tril(A,-1)+eye(n); % siehe A8.1
    
    Y = VorwaertsEinsetzen(L,B);
    X = RueckwaertsEinsetzen(R,Y);
else
    %% Loesung durch Rueckwaertseinsetzen
    R = A;
    C = B;
    X = RueckwaertsEinsetzen(R,C);
end
tElapsed = toc(tStart); % Ende Zeitmessung
end